package com.cultural.heritage.controller.good;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonBatchRequest;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.category.CategoryAddRequest;
import com.cultural.heritage.model.dto.category.CategoryUpdateRequest;
import com.cultural.heritage.model.entity.Category;
import com.cultural.heritage.model.entity.Good;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.CategoryService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/category")
@Slf4j
@Tag(name = "商品类别管理模块")
public class CategoryController {


    @Resource
    private CategoryService categoryService;


    @Resource
    private GoodService goodService;


    @Resource
    private UserService userService;


    @Resource
    private CommonService commonService;


    /**
     * 添加商品类别
     * @param categoryAddRequest  类别添加请求体
     * @return  是否添加成功
     */
    @PostMapping("/add")
    @Operation(summary = "Web端管理员添加商品类别", description = "参数：类别添加请求体，权限：管理员(admin, boss)，方法名：addCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> addCategory(@RequestBody CategoryAddRequest categoryAddRequest) {
        if (categoryAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Category category = new Category();
        BeanUtils.copyProperties(categoryAddRequest, category);
        categoryService.validCategory(category, true);
        boolean save = categoryService.save(category);
        ThrowUtils.throwIf(!save, ErrorCode.OPERATION_ERROR);
        return ResultUtils.success(true, "类别插入成功");
    }


    /**
     * 删除商品类别
     * @param deleteCategoryRequest 类别删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delete")
    @Operation(summary = "Web端管理员删除商品类别", description = "参数：类别删除请求体，权限：管理员(admin, boss)，方法名：deleteCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> deleteCategory(@RequestBody CommonRequest deleteCategoryRequest) {
        if (deleteCategoryRequest == null || deleteCategoryRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 批量删除当前类别下的所有商品
        Long id = deleteCategoryRequest.getId();
        Category category = categoryService.getCategoryById(id);

        String typeName = category.getTypeName();
        List<Good> goodList = commonService.findByFieldEqTargetField("type", typeName, goodService);

        goodService.removeBatchByIds(goodList);
//        ThrowUtils.throwIf(!isSuccess, ErrorCode.OPERATION_ERROR, "商品批量删除失败");

        // 删除当前类别
        boolean result = categoryService.removeById(id);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "类别删除失败");

        return ResultUtils.success(true);
    }


    /**
     * 批量删除商品类别
     * @param commonDelBatchRequest    类别删除请求体
     * @return  是否删除成功
     */
    @PostMapping("/delBatch")
    @Operation(summary = "Web端管理员批量删除商品类别", description = "参数：类别删除请求体，权限：管理员(admin, boss)，方法名：deleteCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> delBatchCategory(@RequestBody CommonBatchRequest commonDelBatchRequest) {
        if (commonDelBatchRequest == null || CollectionUtils.isEmpty(commonDelBatchRequest.getIdList())) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        List<Long> idList = commonDelBatchRequest.getIdList();
        // 批量删除当前类别下的所有商品
        List<Category> categoryList = commonService.findByFieldInTargetField(idList, categoryService, id -> id, "id");
        List<Good> goodList = commonService.findByFieldInTargetField(categoryList, goodService, Category::getTypeName, "type");

        // 批量删除商品
        goodService.removeBatchByIds(goodList);
//        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "商品批量删除失败");

        // 批量删除当前类别
        boolean remove = categoryService.removeBatchByIds(categoryList);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "类别批量删除失败");

        return ResultUtils.success(true, "批量删除类别成功");
    }




    /**
     * Web端管理员更新商品类别
     * @param categoryUpdateRequest  类别更新请求体
     * @return 是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新商品类别", description = "参数：类别更新请求体，权限：管理员(admin, boss)，方法名：updateCategory")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateCategory(@RequestBody CategoryUpdateRequest categoryUpdateRequest) {
        if (categoryUpdateRequest == null || categoryUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 获取原有的类别名称
        Long id = categoryUpdateRequest.getId();
        Category originCategory = categoryService.getById(id);
        String originTypeName = originCategory.getTypeName();
        // 获取目标的类别名称
        String targetTypeName = categoryUpdateRequest.getTypeName();
        // 判空
        if (StringUtils.isAnyBlank(originTypeName, targetTypeName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "类别参数错误");
        }
        // 更新商品类别
        Category category = new Category();
        BeanUtils.copyProperties(categoryUpdateRequest, category);
        categoryService.validCategory(category, false);
        boolean result = categoryService.updateById(category);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "商品类别更新失败");

        // 修改原有类别下所有商品的类名
        UpdateWrapper<Good> goodUpdateWrapper = new UpdateWrapper<>();
        goodUpdateWrapper.eq("type", originTypeName).set("type", targetTypeName);
        goodService.update(goodUpdateWrapper);
//        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "商品类名更新失败");
        return ResultUtils.success(true, "类别更新成功");
    }


    /**
     * Web端管理员查询商品类别
     * @return  商品类别列表
     */
    @PostMapping("/list/web")
    @Operation(summary = "Web端管理员查询商品类别", description = "参数：无，权限：所有人，方法名：listCategoryOnWeb")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<List<Category>> listCategoryOnWeb() {
        List<Category> list = categoryService.list();
        return ResultUtils.success(list);
    }




    /**
     * 小程序端用户查询商品类别
     * @return  商品类别列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询商品类别", description = "参数：无，权限：所有人，方法名：listCategory")
    public BaseResponse<List<Category>> listCategory(HttpServletRequest request) {
        userService.getLoginUser(request);
        // 获取所有已上架的商品
        List<Good> goodList = commonService.findByFieldEqTargetField("isShelves", 1, goodService);
        // 获取所有存在上架商品的商品类别
        List<Category> categoryList = commonService.findByFieldInTargetField(goodList, categoryService, Good::getType, "typeName");
        return ResultUtils.success(categoryList);
    }



    /**
     * 小程序端用户根据类别id查询该类的所有商品
     * @param categoryQueryRequest  类别查询请求体
     * @return 当前类别的商品列表
     */
    @PostMapping("/list/type")
    @Operation(summary = "小程序端用户根据类别id查询该类的所有商品", description = "参数：类别id，方法名：listGoodByCategory")
    public BaseResponse<Map<Long, List<Good>>> listGoodByCategory(@RequestBody CommonRequest categoryQueryRequest, HttpServletRequest request) {
        if (categoryQueryRequest == null || categoryQueryRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long id = categoryQueryRequest.getId();
        Category category = categoryService.getCategoryById(id);
        String typeName = category.getTypeName();
        if (StringUtils.isBlank(typeName)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        QueryWrapper<Good> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("type", typeName);
        queryWrapper.eq("isShelves", 1);
        queryWrapper.orderByDesc("id");
        List<Good> goodList = goodService.list(queryWrapper);
        Map<Long, List<Good>> map = new HashMap<>();
        map.put(id, goodList);
        return ResultUtils.success(map);
    }




}
