package com.cultural.heritage.controller.order;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cultural.heritage.annotation.AuthCheck;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.OrderStatusConstant;
import com.cultural.heritage.constant.UserConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.exception.ThrowUtils;
import com.cultural.heritage.model.dto.CommonRequest;
import com.cultural.heritage.model.dto.cart.CartOrderAddRequest;
import com.cultural.heritage.model.dto.cart.CartOrderItemAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderAddRequest;
import com.cultural.heritage.model.dto.cartService.CartExperienceOrderItemAddRequest;
import com.cultural.heritage.model.dto.order.OrderQueryRequest;
import com.cultural.heritage.model.dto.order.OrderStatusUpdateRequest;
import com.cultural.heritage.model.dto.order.OrderUpdateRequest;
import com.cultural.heritage.model.dto.order.capital.GeneralGoodSingleBuyAddRequest;
import com.cultural.heritage.model.dto.order.capital.OrderMainInfoAddRequest;
import com.cultural.heritage.model.dto.snapshot.CouponSnapshot;
import com.cultural.heritage.model.dto.snapshot.GoodSnapshot;
import com.cultural.heritage.model.entity.*;
import com.cultural.heritage.model.vo.order.OrderVO;
import com.cultural.heritage.service.common.CommonService;
import com.cultural.heritage.service.good.CartExperienceService;
import com.cultural.heritage.service.good.CartRecordService;
import com.cultural.heritage.service.good.GoodService;
import com.cultural.heritage.service.good.UserCouponService;
import com.cultural.heritage.service.order.OrderItemService;
import com.cultural.heritage.service.order.OrderService;
import com.cultural.heritage.service.order.PendingServiceOrderService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/order")
@Slf4j
@Tag(name = "订单管理模块")
@Transactional(rollbackFor = Exception.class)
public class OrderController {

    @Resource
    private OrderService orderService;



    @Resource
    private OrderItemService orderItemService;



    @Resource
    private UserService userService;



    @Resource
    private CartRecordService cartRecordService;



    @Resource
    private CartExperienceService cartExperienceService;



    @Resource
    private GoodService goodService;



    @Resource
    private UserCouponService userCouponService;



    @Resource
    private PendingServiceOrderService pendingServiceOrderService;



    @Resource
    private CommonService commonService;




    /**
     * 小程序端用户创建常规类（服务类）商品订单
     * @param generalGoodSingleBuyAddRequest 常规类商品单独购买创建请求体
     * @return  是否创建成功
     */
    @PostMapping("/add")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端用户创建常规类（服务类）商品订单", description = "参数：订单创建请求体，权限：所有人，方法名：addOrder")
    public BaseResponse<Long> addOrder(@RequestBody GeneralGoodSingleBuyAddRequest generalGoodSingleBuyAddRequest, HttpServletRequest request) {
        if (generalGoodSingleBuyAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验用户当前是否处于登录态
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();

        // 判断是否是常规类
        boolean isGeneral = orderService.isGeneralGood(generalGoodSingleBuyAddRequest.getOrderType());

        // 校验单独购买的商品数据是否准确
        orderService.validSingleGoodOrder(generalGoodSingleBuyAddRequest, isGeneral);

        // 封装成订单主要信息请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = new OrderMainInfoAddRequest();
        BeanUtils.copyProperties(generalGoodSingleBuyAddRequest, orderMainInfoAddRequest);

        // 计算单个商品购买的总金额
        BigDecimal totalAmount = orderService.calculateTotalAmount(orderMainInfoAddRequest);
        orderMainInfoAddRequest.setTotalAmount(totalAmount);
        // 创建通用订单(常规类和服务类商品)
        Long orderId = orderService.createCommonOrder(orderMainInfoAddRequest, userId, false, null, isGeneral);

        // 向消息队列中发送订单创建的消息
        orderService.sendCreateOrderMessage(orderId);

        return ResultUtils.success(orderId);
    }





    /**
     * 小程序端用户创建常规类商品购物车订单
     */
    @PostMapping("/add/cart")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端用户创建常规类商品购物车订单", description = "参数：购物车订单创建请求体，权限：所有人，方法名：addCartOrder")
    public BaseResponse<Long> addCartOrder(@RequestBody CartOrderAddRequest cartOrderAddRequest, HttpServletRequest request) {
        if (cartOrderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验用户当前是否处于登录态
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();

        // 校验购物车数据是否准确
        List<CartOrderItemAddRequest> cartOrderItemAddRequestList = cartOrderAddRequest.getCartOrderItemAddRequestList();
        boolean isAccurate = cartRecordService.validIsConsistent(cartOrderItemAddRequestList);
        ThrowUtils.throwIf(!isAccurate, ErrorCode.OPERATION_ERROR, "当前购物车中的某些商品已下架或者库存不足，无法生成订单");

        // 封装成订单主要信息请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = orderService.packagedOrderMainInfoAddRequest(cartOrderAddRequest);

        // 创建通用订单(常规类和服务类商品)
        List<Long> cartIds = cartOrderItemAddRequestList.stream().map(CartOrderItemAddRequest::getCartRecordId).toList();
        Long orderId = orderService.createCommonOrder(orderMainInfoAddRequest, userId, true, cartIds, true);

        // 向消息队列中发送订单创建的消息
        orderService.sendCreateOrderMessage(orderId);

        return ResultUtils.success(orderId);
    }




    /**
     * 小程序端用户创建服务类商品购物车订单
     */
    @PostMapping("/add/experience/cart")
    @Transactional(rollbackFor = Exception.class)
    @Operation(summary = "小程序端用户创建服务类商品购物车订单", description = "参数：购物车订单创建请求体，权限：所有人，方法名：addCartOrder")
    public BaseResponse<Long> addCartExperienceOrder(@RequestBody CartExperienceOrderAddRequest cartExperienceOrderAddRequest, HttpServletRequest request) {
        if (cartExperienceOrderAddRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        // 校验用户当前是否处于登录态
        User loginUser = userService.getLoginUser(request);
        Long userId = loginUser.getId();

        // 校验购物车数据是否准确
        List<CartExperienceOrderItemAddRequest> cartExperienceOrderItemAddRequestList = cartExperienceOrderAddRequest.getCartOrderItemAddRequestList();
        boolean isAccurate = cartExperienceService.validIsConsistent(cartExperienceOrderItemAddRequestList);
        ThrowUtils.throwIf(!isAccurate, ErrorCode.OPERATION_ERROR, "当前购物车中的某些商品已下架或者不存在，无法生成订单");

        // 封装成订单主要信息请求体
        OrderMainInfoAddRequest orderMainInfoAddRequest = orderService.packagedOrderMainInfoAddRequest(cartExperienceOrderAddRequest);

        // 创建通用订单(常规类和服务类商品)
        List<Long> cartIds = cartExperienceOrderItemAddRequestList.stream().map(CartExperienceOrderItemAddRequest::getCartExperienceId).toList();
        Long orderId = orderService.createCommonOrder(orderMainInfoAddRequest, userId, true, cartIds, false);

        // 向消息队列中发送订单创建的消息
        orderService.sendCreateOrderMessage(orderId);

        return ResultUtils.success(orderId);

    }



    /**
     * Web端管理员分页查询订单
     * @param orderQueryRequest  订单查询请求体
     * @return 订单列表信息
     */
    @PostMapping("/list/page")
    @Operation(summary = "Web端管理员分页查询订单", description = "参数：订单查询请求体，权限：管理员(admin, boss)，方法名：listOrder")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Page<OrderVO>> listOrder(@RequestBody OrderQueryRequest orderQueryRequest) {
        if (orderQueryRequest == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        long current = orderQueryRequest.getCurrent();
        long pageSize = orderQueryRequest.getPageSize();
        Page<Order> page = orderService.page(new Page<>(current, pageSize),
                orderService.getQueryWrapper(orderQueryRequest));
        List<Order> orders = page.getRecords();
        List<OrderVO> orderVOS = commonService.convertList(orders, OrderVO.class);

        // 封装map集合（键：订单id, 值：订单明细列表）
        Map<Long, List<OrderItems>> map = new HashMap<>();
        List<OrderItems> orderItemsList = orderItemService.list();
        for (OrderItems orderItems : orderItemsList) {
            Long orderId = orderItems.getOrderId();

            List<OrderItems> orderItemsArrayList = map.get(orderId);
            if (orderItemsArrayList == null) {
                orderItemsArrayList = new ArrayList<>();
            }
            orderItemsArrayList.add(orderItems);
            map.put(orderId, orderItemsArrayList);
        }

        for (OrderVO orderVO : orderVOS) {
            Long id = orderVO.getId();
            List<OrderItems> orderItemsTempList = map.get(id);
            orderVO.setOrderItemList(orderItemsTempList);
        }

        Page<OrderVO> orderVOPage = new Page<>();
        orderVOPage.setRecords(orderVOS);
        orderVOPage.setTotal(page.getTotal());
        orderVOPage.setPages(page.getPages());
        orderVOPage.setCurrent(page.getCurrent());
        orderVOPage.setSize(page.getSize());
        return ResultUtils.success(orderVOPage);
    }







    /**
     * 小程序端用户查询订单
     * @return  用户订单列表
     */
    @PostMapping("/list")
    @Operation(summary = "小程序端用户查询订单", description = "参数：用户id请求体，权限：所有人，方法名：listUserOrder")
    public BaseResponse<List<OrderVO>> listUserOrder(HttpServletRequest request) {
        User loginUser = userService.getLoginUser(request);
        Long id = loginUser.getId();
        QueryWrapper<Order> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("userId", id);
        queryWrapper.orderByDesc("id");
        List<Order> orders = orderService.list(queryWrapper);

        // 获取订单明细
        List<OrderItems> myOrderItems = commonService.findByFieldInTargetField(orders, orderItemService, Order::getId, "orderId");

        // 封装map集合（键：订单id, 值：订单明细列表）
        Map<Long, List<OrderItems>> map = new HashMap<>();
        for (OrderItems orderItems : myOrderItems) {
            Long orderId = orderItems.getOrderId();

            List<OrderItems> orderItemsArrayList = map.get(orderId);
            if (orderItemsArrayList == null) {
                orderItemsArrayList = new ArrayList<>();
            }
            orderItemsArrayList.add(orderItems);
            map.put(orderId, orderItemsArrayList);
        }

        List<OrderVO> orderVOS = orders.stream().map(order -> {
            Long orderId = order.getId();
            List<OrderItems> orderItemsList = map.get(orderId);
            OrderVO orderVO = new OrderVO();
            BeanUtils.copyProperties(order, orderVO);
            orderVO.setOrderItemList(orderItemsList);
            return orderVO;
        }).toList();
        return ResultUtils.success(orderVOS);
    }


    /**
     * Web端管理员更新物流单号
     * @param orderUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update")
    @Operation(summary = "Web端管理员更新物流单号", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateOrderStatus(@RequestBody OrderUpdateRequest orderUpdateRequest) {
        if (orderUpdateRequest == null || orderUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderUpdateRequest.getId();
        String trackingNumber = orderUpdateRequest.getTrackingNumber();
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("trackingNumber", trackingNumber);
        updateWrapper.set("orderStatus", OrderStatusConstant.PENDING_DELIVERY);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单状态不存在或订单状态更新失败");
        return ResultUtils.success(true);
    }


    /**
     * 小程序端根据订单id查询订单信息
     * @param commonRequest     根据id查询请求体
     * @return   用户订单信息
     */
    @PostMapping ("/get/id")
    @Operation(summary = "小程序端用户根据id查询订单", description = "参数：根据id查询请求体，权限：所有人，方法名：getOrderById")
    public BaseResponse<OrderVO> getOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        userService.getLoginUser(request);
        Long orderId = commonRequest.getId();
        Order order = orderService.getById(orderId);
        ThrowUtils.throwIf(order == null, ErrorCode.NOT_FOUND_ERROR, "订单不存在");
        OrderVO orderVO = new OrderVO();
        BeanUtils.copyProperties(order, orderVO);
        List<OrderItems> orderItemsList = commonService.findByFieldEqTargetField("orderId", orderId, orderItemService);
        orderVO.setOrderItemList(orderItemsList);

        return ResultUtils.success(orderVO);
    }



    /**
     * 小程序端用户取消常规类商品订单
     * @param commonRequest     订单id
     * @return   是否取消成功
     */
    @PostMapping ("/cancel/id")
    @Operation(summary = "小程序端用户取消服务类商品订单", description = "参数：订单id，权限：所有人，方法名：cancelOrderById")
    public BaseResponse<Boolean> cancelOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        Order order = orderService.getById(id);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ThrowUtils.throwIf(!order.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.SYSTEM_ERROR, "订单状态错误");
        order.setOrderStatus(OrderStatusConstant.TRANSACTION_CLOSED);
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");

        // 恢复商品库存
        List<OrderItems> orderItemsList = commonService.findByFieldEqTargetField("orderId", id, orderItemService);
        // 获取商品id列表
        List<Long> goodIds = orderItemsList.stream().map(orderItems -> {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            return goodSnapshot.getId();
        }).toList();
        // 获取商品列表
        List<Good> goodList = commonService.findByFieldInTargetField(goodIds, goodService, g -> g, "id");
        // 封装为map集合（键：商品id, 值：库存量）,用来存储购买的商品id对应的购买数量
        Map<Long, Integer> map = new HashMap<>();
        for (OrderItems orderItems : orderItemsList) {
            GoodSnapshot goodSnapshot = orderItems.getGoodSnapshot();
            Long goodId = goodSnapshot.getId();
            Integer quantity = orderItems.getQuantity();
            map.put(goodId, quantity);
        }

        // 批量更新商品库存
        for (Good good : goodList) {
            Long goodId = good.getId();
            // 已经下单购买的数量
            Integer quantity = map.get(goodId);
            // 原有的库存
            Integer inventory = good.getInventory();
            good.setInventory(quantity + inventory);
        }
        boolean updateBatch = goodService.updateBatchById(goodList);
        ThrowUtils.throwIf(!updateBatch, ErrorCode.SYSTEM_ERROR, "商品库存恢复失败");

        // 如果使用了优惠券，则退还优惠券
        CouponSnapshot couponSnapshot = order.getCouponSnapshot();
        if (couponSnapshot != null) {
            Long userCouponId = couponSnapshot.getId();
            UserCoupon userCoupon = userCouponService.getById(userCouponId);
            ThrowUtils.throwIf(userCoupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
            // 更新优惠券状态
            userCoupon.setIsUsed(0);
            boolean result = userCouponService.updateById(userCoupon);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券状态更新失败");
        }

        return ResultUtils.success(true);
    }




    /**
     * 小程序端用户删除商品订单
     * @param commonRequest     订单id
     * @return   是否取消成功
     */
    @PostMapping ("/delete/id")
    @Operation(summary = "小程序端用户删除商品订单", description = "参数：订单id，权限：所有人，方法名：deleteOrderById")
    public BaseResponse<Boolean> deleteOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        Order order = orderService.getById(id);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ThrowUtils.throwIf(!order.getOrderStatus().equals(OrderStatusConstant.TRANSACTION_CLOSED), ErrorCode.SYSTEM_ERROR, "订单状态错误");

        boolean remove = orderService.removeById(id);
        ThrowUtils.throwIf(!remove, ErrorCode.OPERATION_ERROR, "订单删除失败");
        return ResultUtils.success(true);
    }





    /**
     * 小程序端用户取消服务类商品订单
     * @param commonRequest     订单id
     * @return   是否取消成功
     */
    @PostMapping ("/cancel/service/id")
    @Operation(summary = "小程序端用户取消服务类商品订单", description = "参数：订单id，权限：所有人，方法名：cancelOrderById")
    public BaseResponse<Boolean> cancelServiceOrderById(@RequestBody CommonRequest commonRequest, HttpServletRequest request) {
        if (commonRequest == null || commonRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = commonRequest.getId();
        userService.getLoginUser(request);
        Order order = orderService.getById(id);
        ThrowUtils.throwIf(order == null, ErrorCode.OPERATION_ERROR, "订单不存在");
        ThrowUtils.throwIf(!order.getOrderStatus().equals(OrderStatusConstant.PENDING_PAYMENT), ErrorCode.SYSTEM_ERROR, "订单状态错误");
        order.setOrderStatus(OrderStatusConstant.TRANSACTION_CLOSED);
        boolean update = orderService.updateById(order);
        ThrowUtils.throwIf(!update, ErrorCode.OPERATION_ERROR, "订单状态更新失败");

        // 更新服务类商品待处理订单记录
        List<PendingServiceOrder> pendingServiceOrderList = commonService.findByFieldEqTargetField("orderNumber", order.getOrderNumber(), pendingServiceOrderService);
        for (PendingServiceOrder pendingServiceOrder : pendingServiceOrderList) {
            pendingServiceOrder.setOrderItemStatus(order.getOrderStatus());
        }
        boolean updateResult = pendingServiceOrderService.updateBatchById(pendingServiceOrderList);
        ThrowUtils.throwIf(!updateResult, ErrorCode.OPERATION_ERROR, "服务类商品待处理订单记录更新失败");


        // 如果使用了优惠券，则退还优惠券
        CouponSnapshot couponSnapshot = order.getCouponSnapshot();
        if (couponSnapshot != null) {
            Long userCouponId = couponSnapshot.getId();
            UserCoupon userCoupon = userCouponService.getById(userCouponId);
            ThrowUtils.throwIf(userCoupon == null, ErrorCode.OPERATION_ERROR, "优惠券不存在");
            // 更新优惠券状态
            userCoupon.setIsUsed(0);
            boolean result = userCouponService.updateById(userCoupon);
            ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "优惠券状态更新失败");
        }

        return ResultUtils.success(true);
    }




    /**
     * Web端管理员更新订单状态
     * @param orderStatusUpdateRequest  订单状态更新请求体
     * @return  是否更新成功
     */
    @PostMapping("/update/all/orderStatus")
    @Operation(summary = "Web端管理员更新订单状态", description = "参数：订单状态更新请求体，权限：管理员(admin, boss)，方法名：updateAllOrderStatus")
    @AuthCheck(mustRole = UserConstant.ADMIN_ROLE)
    public BaseResponse<Boolean> updateAllOrderStatus(@RequestBody OrderStatusUpdateRequest orderStatusUpdateRequest) {
        if (orderStatusUpdateRequest == null || orderStatusUpdateRequest.getId() <= 0) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR);
        }
        Long id = orderStatusUpdateRequest.getId();
        String orderStatus = orderStatusUpdateRequest.getOrderStatus();
        ThrowUtils.throwIf(StringUtils.isBlank(orderStatus), ErrorCode.OPERATION_ERROR, "订单状态参数为空");
        UpdateWrapper<Order> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id", id);
        updateWrapper.set("orderStatus", orderStatus);
        boolean result = orderService.update(updateWrapper);
        ThrowUtils.throwIf(!result, ErrorCode.OPERATION_ERROR, "订单状态不存在或订单状态更新失败");
        return ResultUtils.success(true);
    }




}
